%%  This is file `SBU-Thesis.cls',
%%	by: Arash Dargahi Nobari, arash@dargahi.org
%%  http://arashdargahi.com
%%	Ver 2.0.2, July 2019
%% https://github.com/arashdn/sbu-thesis

\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{SBU-Thesis}
[2/7/2019 v2.0.2
Persian thesis document class in XeLaTeX for Shahid Beheshti University projects/theses/dissertations]
\DeclareOption*{\PassOptionsToClass{\CurrentOption}{book}}
\ProcessOptions
\LoadClass[a4paper,oneside,12pt]{book}


\def\university#1{\gdef\@university{#1}}
\def\department#1{\gdef\@department{#1}}
\def\degree#1{\gdef\@degree{#1}}
\def\thesisdate#1{\gdef\@thesisdate{#1}}

\newcommand{\firstsupervisor}[1]{\def\@firstsupervisor{#1}}
\newcommand{\secondsupervisor}[1]{\def\@secondsupervisor{#1}}
\newcommand{\firstadvisor}[1]{\def\@firstadvisor{#1}}
\newcommand{\secondadvisor}[1]{\def\@secondadvisor{#1}}
\def\name#1{\gdef\@name{#1}}
\def\surname#1{\gdef\@surname{#1}}
\def\keywords#1{\gdef\@keywords{#1}}
\def\latinkeywords#1{\gdef\@\latinkeywords{#1}}
\def\fa-abstract#1{\gdef\@fa-abstract{#1}}
\def\acknowledge#1{\gdef\@acknowledge{#1}}
\def\taghdim#1{\gdef\@taghdim{#1}}
\def\pish#1{\gdef\@pish{#1}}
\def\en-abstract#1{\gdef\@en-abstract{#1}}
\def\faculty#1{\gdef\@faculty{#1}}
\def\subject#1{\gdef\@subject{#1}}



\def\davaranSupervisor#1{\gdef\@davaranSupervisor{#1}}
\def\davaranSecondSupervisor#1{\gdef\@davaranSecondSupervisor{#1}}
\def\davaranAdvisor#1{\gdef\@davaranAdvisor{#1}}
\def\davaranInternal#1{\gdef\@davaranInternal{#1}}
\def\davaranExternal#1{\gdef\@davaranExternal{#1}}
\def\davaranAssignee#1{\gdef\@davaranAssignee{#1}}
\def\davaranDate#1{\gdef\@davaranDate{#1}}

%% Original Copyright of AUT-Thesis
%%  This is file `AUT-Thesis.cls',
%%  It is a modification  of Tabriz_thesis of Vahid Damanafshan,
%%	by: Mahmood Amintoosi, m.amintoosi@gmail.com
%%  http://profsite.sttu.ac.ir/mamintoosi
%%	Ver 0.1, Jan 2012

%nkhComment
\def\reportVersion#1{\gdef\@reportVersion{#1}}
\def\field#1{\gdef\@field{#1}}
\def\department#1{\gdef\@department{#1}}
\def\projectLabel#1{\gdef\@projectLabel{#1}}
\def\studentID#1{\gdef\@studentID{#1}}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\def\latintitle#1{\gdef\@latintitle{#1}}
\def\latinauthor#1{\gdef\@latinauthor{#1}}
\def\latindegree#1{\gdef\@latindegree{#1}}
\def\latinfaculty#1{\gdef\@latinfaculty{#1}}
\def\latinthesisdate#1{\gdef\@latinthesisdate{#1}}
\def\latinsubject#1{\gdef\@latinsubject{#1}}
\def\latinfield#1{\gdef\@latinfield{#1}}
\def\firstlatinsupervisor#1{\gdef\@firstlatinsupervisor{#1}}
\def\secondlatinsupervisor#1{\gdef\@secondlatinsupervisor{#1}}
\def\firstlatinadvisor#1{\gdef\@firstlatinadvisor{#1}}
\def\secondlatinadvisor#1{\gdef\@secondlatinadvisor{#1}}
\def\latinname#1{\gdef\@latinname{#1}}
\def\latinsurname#1{\gdef\@latinsurname{#1}}
\def\en-abstract#1{\gdef\@en-abstract{#1}}
\def\latinkeywords#1{\gdef\@latinkeywords{#1}}
\def\latinuniversity#1{\gdef\@latinuniversity{#1}}
%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%پایان‌نامه ارشد
\newif\if@mscthesis
\@mscthesisfalse
\DeclareOption{msc}{\@mscthesistrue}

%%%سمینار ارشد
\newif\if@mscthesisReport
\@mscthesisReportfalse
\DeclareOption{mscS}{\@mscthesisReporttrue}

\ProcessOptions

\ifx\@projectLabel\undefined%
{
	\if@mscthesis
	{\projectLabel{پایان‌نامه کارشناسی ارشد}}
	\else
	\if@mscthesisReport
	{\projectLabel{گزارش سمینار کارشناسی ارشد }}
	\else
	{\projectLabel{رساله}}
	\fi\fi
}\fi

\ifx\@degree\undefined%
{
	\if@mscthesis
	\degree{کارشناسی ارشد}
	\else
	\if@mscthesisReport
	\degree{کارشناسی ارشد}
	\else
	\degree{دکتری}
	\fi\fi
}


%   Reduce the margin of the summary:
\def\changemargin#1#2{\list{}{\rightmargin#2\leftmargin#1}\item[]}
\let\endchangemargin=\endlist 

%   Generate the environment for the abstract:
\newcommand\summaryname{}
\newenvironment{Abstract}%
{\small\begin{center}%
		\bfseries{\summaryname} \end{center}}


%%%%%%%%%%%%%%%%%% صفحه اول پایان‌نامه
\newcommand{\firstPage}{
	\begin{titlepage}
		%\topmargin=-30pt
		\vspace{-10mm}
		\centerline{{\includegraphics[width=4cm]{sbu_logo}}}
		\vspace{2mm}
		%nkhComment: University name may be included in logo
		\ifx\@university\undefined%
		\relax
		\else
		\centerline{\large \bf \@university} 
		\fi
		\vskip -0.5cm
		{\large\bfseries\@faculty\space}
		\vskip 1cm
		%nkhComment: in case the title is too large

		\if@mscthesis 
		{\large\bfseries{\@projectLabel\ \ \@subject}
			\vspace{3mm}
			\ifx\@field\undefined%
			\relax
			\else
			\\گرایش \@field
			\fi
		}
		\else
		\if@mscthesisReport 
		{\large\bfseries{\@projectLabel\ \ \@subject}
			\ifx\@field\undefined%
			\relax
			\else
			\\گرایش \@field
			\fi
		}
		\else
		{\large\bfseries \@projectLabel\ \ \writtenfor
			\@degree{} 
		}
		{\large\bfseries{در رشته \@subject}}
		\vskip .2cm
		\centerline{\large \bfseries%
			گرایش \@field}
		\fi\fi	
		\par
		\vskip 10mm
		
		\linespread{1.2}
		{\huge \titlefont \@title \par}
		\vskip 15mm
		
		{\large\by}%\par
		\\ \vskip 0.45cm
		{\large\bfseries{\large\bfseries \@name\ \ \@surname}}
		%\par
		\vskip 10mm
		\large
		{
			\ifx\@firstsupervisor\undefined%
			\ifx\@secondsupervisor\undefined%
			\else
			\fi
			\else
			\ifx\@secondsupervisor\undefined%
			استاد راهنما:
			\\ \vskip 0.35cm
			{\large\bfseries
				\@firstsupervisor}
			\else
			\large
			اساتید راهنما
			\\ \vskip 0.35cm
			{\large\bfseries \@firstsupervisor{} و \@secondsupervisor}
			\fi\fi}
		\par\large
		{\ifx\@firstadvisor\undefined%
			\ifx\@secondadvisor\undefined%
			\else\fi
			\else
			\ifx\@secondadvisor\undefined%
			\vskip 1cm
			استاد مشاور
			\\ \vskip 0.35cm
			{\large\bfseries
				\@firstadvisor}
			\else
			
			\large
			\vskip 1cm
			استادان مشاور
			\\ \vskip 0.4cm
			{\large\bfseries\@firstadvisor{} و \@secondadvisor}
			\fi\fi}
		
		\par
		\vskip 10mm
		%\vfill
		{\large \@thesisdate}
		%\vfill
	\end{titlepage}%
} % end of firstPage



\def\titlepage{\newpage\centering
	\thispagestyle{empty}
	\parindent 0pt \parskip 10pt plus 1fil minus 1fil
	\def\baselinestretch{1}\@normalsize\vbox to \vsize\bgroup\vbox to 9in\bgroup}
\def\endtitlepage{\par\kern 0pt\egroup\vss\egroup\newpage}
%%%%%%%%%%%%%%%%%%%%%%%

\newcommand\TaghdimPage{
	\newpage
	\thispagestyle{plain}
	\vskip 10cm
	\par
	\@taghdim
	\par
	\newpage
}

%%%%%%%%%%%%%%%%%% صفحه سپاس‌گزاری
\newcommand\acknowledgement
{
	\newpage
	\thispagestyle{plain}
	\vskip 15mm
	{\noindent\bfseries \hfil \large سپاس‌گزاری}
	\vspace{5mm}
	\par
	\noindent
	\@acknowledge
	\vspace{8mm}
	\par
	\newpage
}

%%%%%%%%%%%%%%%%%% صفحه چکیده
\newcommand\abstractPage
{
	\newpage
	\thispagestyle{plain}
	\vskip 15mm
	{\noindent\bfseries \hfil \large چکیده}
	\vspace{5mm}
	\par
	\noindent
	\@fa-abstract
	\vspace{8mm}
	\par
	\noindent
	\ifdefined\@keywords
	{
		\textbf{واژگان کلیدی:} 
		\@keywords
	}
	\fi
	\newpage
}
%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%% صفحه پیشگفتار
\newcommand\prefacePage
{
	\newpage
	\thispagestyle{plain}
	\vskip 15mm
	{\noindent\bfseries \hfil \large پیشگفتار}
	\vspace{5mm}
	\par
	\noindent
	\@pish
	\vspace{8mm}
	\par
	\noindent
	\ifdefined\@keywords
	\fi
	%\newpage
}
%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%% صفحه چکیده انگلیسی
\newcommand{\latinAbstractPage}
{
	\newpage\clearpage
	\thispagestyle{empty}
	\vskip 15mm
	{\noindent\bfseries \hfil Abstract}
	\vspace{5mm}
	\par
	\noindent
	\begin{LTR}
		\lr{
			\@en-abstract
		}
		
		\vspace{8mm}
		\par
		%	\noindent
		\ifdefined\@latinkeywords
		{
			\lr{
				\textbf{Keywords:} 
				\@latinkeywords
			}
		}
		\fi
	\end{LTR}
}
%%%%%%%%%%%%%%%%


%%%%%%%%%%%%% صفحه عنوان انگلیسی
\newcommand{\latinFirstPage}
{
	\begin{titlepage}
		\linespread{0.8}
		\vspace{-10mm}
		\centerline{{\includegraphics[width=4cm]{sbu_logo}}}
		%nkhComment: University name may be included in logo
		\vspace{2mm}
		\ifx\@latinuniversity\undefined%
		\relax
		\else
		\centerline{\large \lr{\@latinuniversity}} 
		\fi
		%\vskip 2mm
		\large
		\lr{\@latinfaculty}\space 
		\vskip 1cm
		%nkhComment: in case the title is too large

		{
			\large
			
			\lr{
				A THESIS SUBMITTED\\
				FOR THE DEGREE OF\\
			}
			\if@mscthesis
			\lr{MASTER OF SCIENCE}
			\else
			\if@mscthesisReport
			\lr{MASTER OF SCIENCE}
			\else
			\lr{Doctor of Philosophy}
			\fi \fi
			%			in  \@latinsubject 
			%\baselineskip=.750cm
			%\addtolength{\baselineskip}{-5mm}
			%			\fi
			\par
		}
		
		\vskip 10mm
		
		\linespread{1.2}
		{%\latintitlefont 
			\huge
			\bf \lr{\@latintitle} \par}
		%\baselineskip=.750cm \par
		%\linespread{1.2}
		
		\vskip 15mm
		
		{\large By:}\par
		\vskip 0.45cm
		{\large 
			\rm \bf \large \lr{\@latinname \; \@latinsurname}}\par
		
		\vskip 10mm
		
		\large
		{\ifx\@firstlatinsupervisor\undefined%
			\ifx\@secondlatinsupervisor\undefined%
			\else\fi
			\else
			\ifx\@secondlatinsupervisor\undefined%
			\large\lr{Supervisor:}
			\vskip 5mm
			{\large\bfseries\par
				\lr{\@firstlatinsupervisor}}
			\else
			\large
			\Large\lr{Supervisors:}
			\vskip 5mm
			\par
			{\large\bfseries \lr{\@firstlatinsupervisor{} and \@secondlatinsupervisor}}
			\fi\fi}
		\par
		\vspace{1cm}
		
		
		%\vspace{1.5cm}
		\large
		{\ifx\@firstlatinadvisor\undefined%
			\ifx\@secondlatinadvisor\undefined%
			\else\fi
			\else
			\ifx\@secondlatinadvisor\undefined%
			\large\lr{Advisor:}
			\vskip 5mm
			{\large\bfseries\par
				\lr{\@firstlatinadvisor}}
			\else
			
			\large
			\Large\lr{Advisors:}
			\vskip 5mm
			\par
			{\Large\bfseries \lr{\@firstlatinadvisor{} and \@secondlatinadvisor}}
			\fi\fi}
		\vskip 5mm
		
		\par
		\vspace{1cm}
		%\vspace*{\fill}
		
		\large{\lr{\@latinthesisdate}}
		
		%
	\end{titlepage}
}
%%%%%%%%%%%%



%\pagenumbering{adadi}
%\let\ol@chapter\@chapter
%\def\@chapter{%
%	\ifnum\c@chapter=0 \pagenumbering{arabic}\setcounter{page}{1}\fi
%	\ol@chapter}


%nkhComment
\def\version{نسخه‌ی }
\def\report{گزارش}
\def\onvan{عنوان}
\def\writtenfor{برای دریافت درجه }
\def\by{نگارنده:}
\def\latinby{By:}
\def\cleardoublepage{\clearpage\if@twoside \ifodd\c@page\else
	\hbox{}
	\thispagestyle{empty}
	\newpage
	\if@twocolumn\hbox{}\newpage\fi\fi\fi}


\newcommand{\cchapter}[1]
{
	\chapter[#1]{\centering #1}
	\thispagestyle{empty} % حذف شماره صفحه
	\pagebreak
}




%%%%%%Commands

%تنظیم حاشیه صفحات
\usepackage[top=35mm, bottom=25mm, left=25mm, right=25mm]{geometry}
\usepackage{lscape}


\usepackage[bottomtitles]{titlesec} % ایجاد فاصله بین عناوین و خط مطلب بعدی
\usepackage{caption}
\usepackage{subcaption}
\usepackage[xindy,acronym,toc]{glossaries}% اضافه کردن مراجع و نمایه به فهرست مطالب

\usepackage[raggedright]{sidecap}
\usepackage[svgnames]{xcolor}
\usepackage{colortbl}
\usepackage{fontspec}
\usepackage{setspace}
\usepackage{minted}
\setmonofont{AzarMehrMonospaced}


%\usepackage{polyglossia}
%\setdefaultlanguage{persian}
%\setotherlanguages{english,greek}

%\titleformat{,〈,command,〉,}[,〈,shape,〉,]{,〈,format,〉,}{,〈,label,〉,}{,〈,sep,〉,}{,〈,before-code,〉,}[,〈,after-code,〉,]
\titleformat{\section}
{\normalfont\Large\bfseries}{\thesection}{1em}{}
\titleformat{\subsection}
{\normalfont\large\bfseries}{\thesubsection}{1em}{}
\titleformat{\subsubsection}
{\normalfont\normalsize\bfseries}{\thesubsubsection}{1em}{}
\titleformat{\paragraph}[runin]
{\normalfont\normalsize\bfseries}{\theparagraph}{1em}{}
\titleformat{\subparagraph}[runin]
{\normalfont\normalsize\bfseries}{\thesubparagraph}{1em}{}

\titlespacing\section{0pt}{22pt plus 4pt minus 2pt}{0pt plus 2pt minus 2pt}
\titlespacing\subsection{0pt}{18pt plus 4pt minus 2pt}{0pt plus 2pt minus 2pt}
\titlespacing\subsubsection{0pt}{16pt plus 4pt minus 2pt}{0pt plus 2pt minus 2pt}

\usepackage{indentfirst}
\usepackage{multirow}
% در ورژن جدید زی‌پرشین برای تایپ متن‌های ریاضی، این سه بسته، حتماً باید فراخوانی شود
\usepackage{amsthm,amssymb,amsmath}
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%تنطیمات خط بالایی صفحه
\usepackage{fancyhdr}
\pagestyle{fancy}
\setlength{\headheight}{15pt}

\lhead{}
% Length to control the \fancyheadoffset and the calculation of \headline
% simultaneously
\newlength\FHoffset
\setlength\FHoffset{0cm} %بیشتر بودن خط بالا از حاشیه کناری صفحه

\addtolength\headwidth{2\FHoffset}

\fancyheadoffset{\FHoffset}

% these lengths will control the headrule trimming to the left and right 
\newlength\FHleft
\newlength\FHright

% here the trimmings are controlled by the user
\setlength\FHleft{1cm}
\setlength\FHright{0cm}

\lhead{\thepage} % نمایش شماره صفحه در سمت راست هدر
\rhead{\leftmark}
\cfoot{} % نمایش ندادن شماره صفحه در پایین صفحه

\renewcommand{\chaptermark}[1]{\markboth{\MakeUppercase{ #1}}{}} %حذف فصل ۱ از عنوان هدر‌ها

% The new definition of headrule that will take into acount the trimming(s)
\newbox\FHline
\setbox\FHline=\hbox{\hsize=\paperwidth%
	\hspace*{\FHleft}%
	\rule{\dimexpr\headwidth-\FHleft-\FHright\relax}{\headrulewidth}\hspace*{\FHright}%
}

\linespread{2} % تنظیم فاصله خطوط به 1.5

%\renewcommand\headrule{\vskip-0.2\baselineskip\copy\FHline}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\usepackage[bottom,hang,flushmargin]{footmisc} % شروع شماره زیرنویس ها از ۱ در هر صفحه و انتقال به پایین
\usepackage{zref-perpage}
\zmakeperpage{footnote}
%\setlength{\footnotemargin}{5pt} % این دستور باعث ایجاد مشکل در پاورقی‌های دو رقمی می‌شود.
\setlength{\skip\footins}{12pt} %تنظیم فاصله متن تا خط زیرنویس


% بسته‌ و دستوراتی برای ایجاد لینک‌های رنگی با امکان جهش
\usepackage[pagebackref=false,colorlinks,linkcolor=blue,citecolor=blue]{hyperref}
% چنانچه قصد پرینت گرفتن نوشته خود را دارید، خط بالا را غیرفعال و  از دستور زیر استفاده کنید چون در صورت استفاده از دستور زیر‌‌، 
% لینک‌ها به رنگ سیاه ظاهر خواهند شد که برای پرینت گرفتن، مناسب‌تر است
%\usepackage[hidelinks,pagebackref=false]{hyperref}
%\usepackage{cleveref}


\setcounter{tocdepth}{3}
% نمایش \subsubsection در فهرست مطالب
%\part = -1, chapter = 0, \section = 1, subsection = 2, \subsubsection = 3, \paragraph = 4, \subparagraph = 5
%\setcounter{secnumdepth}{3}% Number \subsubsection این که بیشتر از سه سطح شماره گذاری داشته باشیم خوانایی را کاهش می‌دهد.

\usepackage{bookmark} % برای اضافه کردن لینک‌های کنار pdf و همینطور لینک شدن رفرنس و زیرنویس‌ها

\input{config}

\usepackage[font=small,labelfont=bf,skip=4pt]{caption}
\usepackage{graphicx}
%\usepackage{subfig}
\usepackage{longtable}
\usepackage{enumitem}

\setlength{\emergencystretch}{4pt} %جلوگیری از نشکستن خطوط

%\usepackage{persian-bib}
%\usepackage[natbib=True]{biblatex}


\usepackage[extrafootnotefeatures]{xepersian}
\settextfont[BoldFont={IRLotusICEE_Bold.ttf}, BoldItalicFont={IRLotusICEE_BoldIranic.ttf}, ItalicFont={IRLotusICEE_Iranic.ttf},Scale=1.2]{IRLotusICEE.ttf}%{IRZar.ttf}

\setlatintextfont[Scale=1]{Times New Roman}

%%%%%%%%%%%%%%%%%%%%%%%%%%
%\setdigitfont[Scale=1.2]{IRLotusICEE.ttf}
%%%%%%%%%%%%%%%%%%%%%%%%%%
% تعریف قلم های فارسی و انگلیسی اضافی برای استفاده در بعضی از قسمت‌های متن
\defpersianfont\titlefont[Scale=1]{XB Titre.ttf}
\setiranicfont[Scale=1.3]{IRLotusICEE_Iranic.ttf}				% ایرانیک، خوابیده به چپ
% \defpersianfont\nastaliq[Contextuals=Swash,Scale=1.2]{IranNastaliq}
%\setmathsfdigitfont{IRTitr.ttf}

%\settextfont{IRLotusICEE}
%\setlatintextfont{IRLotusICEE}
%\defpersianfont\titlefont{IRTitr}


\deflatinfont\Greekfont[Script=Greek]{CMU Serif}
\DeclareTextFontCommand\textgreek{\Greekfont}



\let\oldtoc\tableofcontents
\renewcommand{\tableofcontents}
{
	\pagestyle{plain}
	\oldtoc
	\clearpage
	\pagestyle{fancy}
}

\let\oldlistoffigures\listoffigures
\renewcommand{\listoffigures}
{
	\pagestyle{plain}
	\oldlistoffigures
	\clearpage
	\pagestyle{fancy}
}

\let\oldlistoftables\listoftables
\renewcommand{\listoftables}
{
	\pagestyle{plain}
	\oldlistoftables
	\clearpage
	\pagestyle{fancy}
}


% دستوری برای تغییر نام کلمه «کتاب‌نامه» به «مراجع»
\renewcommand{\bibname}{مراجع}

\renewcommand\listfigurename{فهرست نمودارها}
\renewcommand\listtablename{فهرست جدول‌ها}

%\renewcommand{\bibname}{\normalsize \begin{center} مراجع \end{center}}
% دستوری برای تعریف واژه‌نامه انگلیسی به فارسی
\newcommand\persiangloss[2]{#1\dotfill\lr{#2}\\}
% دستوری برای تعریف واژه‌نامه فارسی به انگلیسی 
\newcommand\englishgloss[2]{#2\dotfill\lr{#1}\\}


%\usepackage{etoolbox}
%\newbool{firstbib}
%\booltrue{firstbib}
%\preto{\bibitem}{
%	\ifbool{firstbib}
%	{
%		\thispagestyle{fancy}
%		\setbool{firstbib}{false}
%	}{}
%}

%\usepackage[resetlabels]{multibib}



\usepackage{notoccite} %جلوگیری از شماره‌گزاری تصاویر در فهرست
\usepackage{cite} % تبدیل ارجاعات به جای چند عدد به محدوده مثلا [10-13]


\let\oldbibliography\bibliography
\renewcommand{\bibliography}[1]
{
	\linespread{1.2}
	\addcontentsline{toc}{chapter}{مراجع}
	\oldbibliography{#1}
	\linespread{2}
}

\usepackage{tikz}
\usetikzlibrary{shapes,arrows,matrix,positioning}

\tikzstyle{terminator} = [rectangle, draw, text centered, rounded corners, minimum height=2em]
\tikzstyle{process} = [rectangle, draw, text centered, minimum height=2em]
\tikzstyle{decision} = [diamond, draw, text centered, minimum height=2em]
\tikzstyle{data}=[trapezium, draw, text centered, trapezium left angle=60, trapezium right angle=120, minimum height=2em]
\tikzstyle{connector} = [draw, -latex']

%% Distributable under the LaTeX Project Public License,
%% version 1.3c or higher (your choice). The latest version of
%% this license is at: http://www.latex-project.org/lppl.txt
%% 
%% 
%% Special thanks to Vafa Khalighi for xepersian package.
%%
%% End of file `SBU-Thesis.cls'.

